/*
 * Copyright 2005-2008 by beck et al. projects GmbH, Munich
 * Copyright 2009 by Kappich Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.ars.ars.
 * 
 * de.bsvrz.ars.ars is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.ars.ars is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.ars.ars; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.ars.ars.mgmt.tasks;

import de.bsvrz.ars.ars.mgmt.ArchiveManager;
import de.bsvrz.ars.ars.mgmt.datatree.DataIdentTree;
import de.bsvrz.ars.ars.mgmt.tasks.tasklistener.RingOfDeathListener;
import de.bsvrz.ars.ars.mgmt.tasks.tasklistener.TaskStepListener;
import de.bsvrz.ars.ars.persistence.PersistenceManager;
import de.bsvrz.sys.funclib.debug.Debug;

import java.util.*;

/**
 * Abstrakte Basisklasse fuer die abstrakten Task-Basisklassen {@link de.bsvrz.ars.ars.mgmt.tasks.SingleTask} und {@link
 * de.bsvrz.ars.ars.mgmt.tasks.MultiTask}.
 *
 * @author beck et al. projects GmbH
 * @author Thomas Schaefer
 * @author Alexander Schmidt
 * @version $Revision: 6419 $ / $Date: 2009-03-10 23:15:57 +0100 (Tue, 10 Mar 2009) $ / ($Author: rs $)
 */
public abstract class Task extends Thread implements SuspendableTask {

	/** Groesse des Byte-Arrays fuer temporaeren Speicher beim Abfragen der Indexe. */
	protected final static int INDEX_MEMORY_SIZE = 16 * 1024;

	protected Debug logger;

	/**
	 * Zeigt an ob der Task beendet wurde. Mit dieser Hilfsvariable gilt stets <p><code> task.terminateTask(); task.isTaskTerminated()==true; </code></p> was
	 * bei <p><code> task.interrupt(); task.isInterrupted()==true; </code></p> nicht der Fall ist.
	 */
	protected boolean isTerminated = false;

	/** Alle Objekte in dieser Liste werden benachrichtigt, wenn ein Zyklus der Hauptschleife durchlaufen wurde. */
	protected List<TaskStepListener> taskStepListeners = new ArrayList<TaskStepListener>();

	/** Zentrale Archiv-Verwaltung. */
	protected ArchiveManager archMgr;

	protected DataIdentTree didTree;

	/** PersistenceManager fuer den Zugriff auf die Daten des Archivsystems */
	protected PersistenceManager persMgr;

	/** Dieses Objekt wird benachrichtigt, wenn die Hauptschleife verlassen wurde */
	protected RingOfDeathListener ringOfDeathListener = null;

	protected Task(ArchiveManager archiveMgr, RingOfDeathListener rodl) {
		this.setName(getClass().getSimpleName());
		ringOfDeathListener = rodl;
		archMgr = archiveMgr;
		if(archMgr != null) {
			didTree = archMgr.getDataIdentTree();
			persMgr = archMgr.getPersMgr();
		}
	}

	/**
	 * Zu Testzwecken kann hiermit ein Listener eingefuegt werden, der am Ende eines jedem Schleifendurchlaufs aufgerufen wird. Auf diese Weise sind z.B. die Tasks
	 * einfacher zu testen. Wenn ein Task abgebrochen wird, kann nicht garantiert werden, dass fuer die letzte Schleife der {@link TaskListener} aufgerufen wird.
	 *
	 * @param tsl Callback-Interface
	 */
	public void addTaskStepListener(TaskStepListener tsl) {
		synchronized(taskStepListeners) {
			taskStepListeners.add(tsl);
		}
	}

	/**
	 * Entfernt den Listener.
	 *
	 * @param tsl Callback-Interface
	 */
	public void removeTaskStepListener(TaskStepListener tsl) {
		synchronized(taskStepListeners) {
			taskStepListeners.remove(tsl);
		}
	}

	/**
	 * Terminiert den Task, indem die {@link #run()}-Methode unterbrochen wird, sofern sie im Wartezustand ist und nicht gerade einen Auftrag bearbeitet.
	 * Andernfalls wird der Task vor dem naechsten Zyklus beendet.
	 */
	public void terminateTask() {
		isTerminated = true;
		interrupt();
	}

	public boolean isTaskTerminated() {
		return isTerminated;
	}

	public ArchiveManager getArchMgr() {
		return archMgr;
	}

	/**
	 * Hlt den Task an, falls sich das Archivsystem in einem kritischen Zustand befindet. Wird das Archivsystem beendet, wird der Task fortgesetzt, so dass er
	 * seine Aufgabe noch vollstndig erfllen kann. {@link #isTerminated} liefert anschlieend <code>true</code> zurck.
	 */
	public void suspendTaskIfNecessary() {
		if(!isTaskTerminated()) {		//nur anhalten, wenn das Archivsystem nicht herunterfahren soll.
			try {
				archMgr.getInQueuesMgr().checkForSuspendNonWriteTask();
			}
			catch(InterruptedException e) {
				isTerminated = true;	//Exception wird geworfen, wenn das Archivsystem herunter fahren soll -> Auftrag fortsetzen, Task erst danach beenden.
			}
		}
	}
		
}
